{******************************************************************************}
{                             FindFile Component                               }
{                                   2-12-96                                    }
{                              Jeroen van Rooij                                }
{                     Copyright (c) 1994 - 1996 QuickSoft                      }
{                     Findprocedure found in Loyd's Delphi                     }
{******************************************************************************}

unit Findvcl;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs;

type
  TFileFoundEvent = procedure(SearchRec: TSearchRec; Directory: String) of object;
  TFindFile = class(TComponent)
  private
    { Private declarations }
    fFilesToFound: TStrings;
    fDirectory: string;
    fRecursive : boolean;
    fOnFileFound: TFileFoundEvent;
    function GetFilesToFound: TStrings;
    procedure SetFilesToFound(Value: TStrings);


  protected
    { Protected declarations }
    procedure FFind;
    function GetRecursive : boolean;
  public
    { Public declarations }
    Searching: boolean;
    procedure Execute;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }

  {******************************************************************************}
  {Bestanden die gezocht moeten worden.                                          }
  {De stringlijst moet ten minste 1  bestandspecificatie bevatten.               }
  {FileToFind.Strings[0] moet dus bijv. *.pas bevatten.                          }
  {******************************************************************************}
   property FilesToFound: TStrings read GetFilesToFound write SetFilesToFound stored True;

   {Recursief zoeken}                                
   property Recursive : boolean Read GetRecursive Write FRecursive;

   {Begin directory}
   property Directory: string read FDirectory write FDirectory;

   {Wat te doen als er een bestand gevonden is}
   property OnFileFound: TFileFoundEvent read FOnFileFound write FOnFileFound;


  end;

procedure Register;

implementation
type CharSet = Set of Char;

procedure Register;
begin
  RegisterComponents('QuickSoft', [TFindFile]);
end;

function TFindFile.GetFilesToFound: TStrings;
begin
  Result := FFilesToFound;
end;

function TFindFile.GetRecursive : boolean;
begin
   GetRecursive := FRecursive;
end;

procedure TFindFile.Execute;
begin
  FFind;
end;

procedure TFindFile.SetFilesToFound(Value: TStrings);
begin
  if Value<>nil then FFilesToFound.Assign(Value);
end;

constructor TFindFile.Create(AOwner: TComponent);
var s: string;
begin
  inherited Create(AOwner);
  FFilesToFound:=TStringList.Create;
  FilesToFound:=nil;
  GetDir(0,s);
  fDirectory:=s;
  fRecursive:=true;

end;

destructor TFindFile.Destroy;
begin
  FFilesToFound.Free;
  inherited Destroy;
end;


function LTrim(s: string): string;
var i: integer;
begin
  i := 1;
  while s[i] = ' ' do inc(i);
  result := copy(s, i, length(s) - i + 1);
end;

function RTrim(s: string): string;
begin
  while s[length(s)] = ' ' do dec(s[0]);
  result := s;
end;

function AllTrim(s: string): string;
begin
  result := rTrim(lTrim(s));
end;


procedure TFindFile.FFind;
 procedure SearchTree;
  var
    SearchRec: TSearchRec;
    DosError: integer;
    dir: string;
    tmpFileName:String;
    i: integer;
    Attr: Word;
  begin
    GetDir(0, dir);
    tmpFileName:='';

    Attr:=faAnyFile;
    if dir[length(dir)] <> '\' then dir := dir + '\';
    if FFilesToFound.Count >0 then
   {Voor alle bestanden in FFilesToFound}
   for I:=0 to Pred(FFilesToFound.Count) do
   begin
      tmpFileName:=AllTrim(FFilesToFound.Strings[i]);
      DosError := FindFirst(tmpFileName, faArchive+faReadOnly, SearchRec);
    while DosError = 0 do
     begin
      if Assigned(FOnFileFound) then FOnFileFound(SearchRec,Dir);
        DosError := FindNext(SearchRec);
    end;
   end;
   if fRecursive then
    begin
    {Alle bestanden gevonden, is er een subdirectory?}
    DosError := FindFirst('*.*', faDirectory, SearchRec);
    while DosError = 0 do begin
      {Ja, zoek verder}
      if ((SearchRec.attr and faDirectory = faDirectory) and
        (SearchRec.name <> '.') and (SearchRec.name <> '..'))
     then
       begin
        ChDir(SearchRec.name);
        SearchTree; {Recursie!}
        ChDir('..'); {Terug naar ouder}
      end;
      DosError := FindNext(SearchRec); {Nog een directory?}
    end;
  end; {SearchTree}
 end;

begin
  Searching:=True;
  ChDir(FDirectory);
  SearchTree;
  Searching:=False;
end; {FileFind}



end.
